#define BLYNK_TEMPLATE_ID "TMPL6QR1MAD7n"
#define BLYNK_TEMPLATE_NAME "Creative Workspace"
#define BLYNK_AUTH_TOKEN "4WCUskLaQ9IJTXFgLJHZLFw_EJEmE_QP"
#define BLYNK_PRINT Serial

/*
    Creative Workspace IoT Project
    This project controls LED brightness and color using a light sensor,
    a temperature sensor to monitor room temperature,
    and a speaker for audio alerts. Controlled via the Blynk app.

    The circuit:
    * Light sensor connected to GPIO 32
    * DHT22 temperature sensor connected to GPIO 26
    * LED strip connected to GPIO 12
    * Speaker connected to GPIO 5

    Created By:
    Yuval Greenshpan 206094310
*/ 

#include <WiFi.h>
#include <WiFiClient.h>
#include <BlynkSimpleEsp32.h>
#include <Adafruit_NeoPixel.h>
#include "DHT.h"

// Pin definitions
#define PIN 12
#define NUMPIXELS 12
#define SPEAKER_PIN 5
#define LIGHT_SENSOR_PIN 32
#define DHTPIN 26
#define DHTTYPE DHT22

// Global variables and objects
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);
DHT dht(DHTPIN, DHTTYPE);
char ssid[] = "SHOOG";
char pass[] = "shoog123";
bool isLedOn = false;
bool isSpeakerOn = false;
int baseFrequency = 300;
int frequencyRange = 200;
int brightness = 255;
unsigned long previousMillis = 0;
const long interval = 50;

// LEDC channel settings
const int ledChannel = 0;
const int ledcTimer = 0;
const int ledcResolution = 8;
const int ledcFrequency = 2000;

void setup() {
  Serial.begin(115200);
  Blynk.begin(BLYNK_AUTH_TOKEN, ssid, pass);
  strip.begin();
  strip.show();
  pinMode(SPEAKER_PIN, OUTPUT);

  // Initialize LEDC
  ledcSetup(ledChannel, ledcFrequency, ledcResolution);
  ledcAttachPin(SPEAKER_PIN, ledChannel);

  // Initialize DHT sensor
  dht.begin();
}

void loop() {
  Blynk.run();
  
  // Read and handle light sensor data
  int lightValue = analogRead(LIGHT_SENSOR_PIN);
  if (isLedOn) {
    changeColorBasedOnLight(lightValue);
  } else {
    for (int i = 0; i < NUMPIXELS; i++) {
      strip.setPixelColor(i, strip.Color(0, 0, 0));
    }
    strip.show();
  }

  // Handle speaker control
  if (isSpeakerOn) {
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis >= interval) {
      previousMillis = currentMillis;
      int currentFrequency = baseFrequency + (frequencyRange * sin((float)currentMillis / 500.0));
      ledcWriteTone(ledChannel, currentFrequency);
    }
  } else {
    ledcWriteTone(ledChannel, 0);
  }

  // Read temperature
  float t = dht.readTemperature();
  if (!isnan(t)) {
    Blynk.virtualWrite(V4, t);
  }
}

// Function to turn on the LED strip
BLYNK_WRITE(V1) {
  int buttonState = param.asInt();
  isLedOn = buttonState == 1;
  if (!isLedOn) {
    for (int i = 0; i < NUMPIXELS; i++) {
      strip.setPixelColor(i, strip.Color(0, 0, 0));
    }
    strip.show();
  }
}

// Function to adjust brightness
BLYNK_WRITE(V2) {
  int sliderValue = param.asInt();
  brightness = map(sliderValue, 0, 10, 0, 255);
  if (isLedOn) {
    int lightValue = analogRead(LIGHT_SENSOR_PIN);
    changeColorBasedOnLight(lightValue);
  }
}

// Function to control the speaker
BLYNK_WRITE(V3) {
  int speakerState = param.asInt();
  isSpeakerOn = (speakerState == 1);
}

// Function to change LED color based on light sensor value
void changeColorBasedOnLight(int lightValue) {
  int red = map(lightValue, 0, 4095, 0, 255) * brightness / 255;
  int green = map(lightValue, 0, 4095, 255, 0) * brightness / 255;
  int blue = map(lightValue, 0, 4095, 0, 255) * brightness / 255;
  for (int i = 0; i < NUMPIXELS; i++) {
    strip.setPixelColor(i, strip.Color(red, green, blue));
  }
  strip.show();
}
